#include "Fog.fx"

cbuffer cbPerFrame
{
	float4x4 gWVP;
	float3 gUVDiff;
};
 
TextureCube gCubeMap;
TextureCube gCloudMap;

DepthStencilState LessEqualDSS
{
    DepthEnable = TRUE;
    DepthWriteMask = ALL;
	DepthWriteMask = 0;
	DepthFunc = LESS_EQUAL;
};

SamplerState gTriLinearSam
{
	Filter = MIN_MAG_MIP_LINEAR;
	AddressU = Wrap;
	AddressV = Wrap;
};

struct VS_IN
{
	float3 posL : POSITION;
};

struct PS_IN
{
	float4 posH : SV_POSITION;
    float3 texC : TEXCOORD0;
};
 
PS_IN VS(VS_IN vsIn)
{
	PS_IN vsOut;
	
	//do the transformation as z = w so after the homogeneous division the distance will be 1
	vsOut.posH = mul(float4(vsIn.posL, 1.0f), gWVP).xyww;

	// use local vertex position as cubemap lookup vector.
	vsOut.texC = vsIn.posL;
	
	return vsOut;
}

float4 PS(PS_IN psIn) : SV_Target
{
	float3x3 rot = {cos(gUVDiff.x)  ,  sin(gUVDiff.x)  ,  0,
					-sin(gUVDiff.x) ,  cos(gUVDiff.x)  ,  0,
					0				,  0			   ,  1};


	float fog = CalcFogFactor( 1300, psIn.texC );
	
	float3 p1 = normalize(psIn.texC);
	float dd = 1-pow(max(dot(p1,float3(0,1,0)),0.0),0.35);
	
	float4 psOut = gCubeMap.Sample(gTriLinearSam, psIn.texC)*0.8;
	
	//moving clouds
	psIn.texC = mul(rot,psIn.texC);
	psOut.rgb += gCloudMap.Sample(gTriLinearSam, psIn.texC).rgb;
	
	psOut.rgb = lerp(g_fogColor, psOut.rgb, fog);
	
	return psOut;
}



technique10 GlobalCubeMapTech
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
        
		SetDepthStencilState(LessEqualDSS, 0);
    }
}
