#include "Fog.fx"
#include "ShadowMap.fx"

cbuffer cbPerFrame
{
	float4x4 gWorld;
	float4x4 gWVP;
	float4x4 gView;
	float4 gDirToSunW;
};

cbuffer cbFixed
{
	float gTexScale = 45;
	float gLayersCount;
};
 

Texture2DArray gDiffuseLayers;
Texture2DArray gBlendMap;
Texture2D gSteepMap;

SamplerState gAnisotropicSam
{
	Filter = ANISOTROPIC;
	AddressU = Wrap;
	AddressV = Wrap;
};

SamplerState gTriLinearSam
{
	Filter = MIN_MAG_MIP_LINEAR;
	AddressU = Wrap;
	AddressV = Wrap;
};

struct VS_IN
{
	float3 posL    : POSITION;
	float3 normalL : NORMAL;
	float2 texC    : TEXCOORD;
};

struct PS_IN
{
	float4 posH         : SV_POSITION;
	float3 normalW		: NORMAL;
	float shade         : SHADE;
    float2 tiledUV      : TEXCOORD0;
    float2 stretchedUV  : TEXCOORD1; 
	float  fogDist		: FOGDISTANCE;
	float4 projTexC		: TEXCOORD2;
};

PS_IN VS(VS_IN vIn)
{
	PS_IN vsOut;
	
	vsOut.posH = mul(float4(vIn.posL, 1.0f), gWVP);
		
	vsOut.normalW = mul(vIn.normalL, (float3x3)gWorld);
	
	//here just vertex lighting (the last is the ambient term)
	vsOut.shade = saturate(max(dot(vsOut.normalW, gDirToSunW), 0.0f) + 0.1f);

	vsOut.tiledUV     = gTexScale*vIn.texC;
	vsOut.stretchedUV = vIn.texC;

	float4 vPosWorld = mul( float4(vIn.posL,1), gWorld );
	
	vsOut.fogDist = mul( vPosWorld, gView ).z;

	vsOut.projTexC = mul(float4(vIn.posL, 1.0f), gLightWVP);

	return vsOut;
}

float4 PS(PS_IN psIn) : SV_Target
{
	
	float3 normalWorld = normalize(psIn.normalW);

	float slope = pow( dot(float3(0.0f, 1.0f, 0.0f), normalWorld), 2 );
	if (slope > 0.75) slope = 1;


	float4 c0;
	float4 C = gDiffuseLayers.Sample(gAnisotropicSam, float3(psIn.tiledUV,0));
	float t;
	int i = 0;
	while (i<gLayersCount) {
		t = gBlendMap.Sample( gTriLinearSam,  float3(psIn.stretchedUV,i) );
		c0 = gDiffuseLayers.Sample(gAnisotropicSam, float3(psIn.tiledUV,++i));
		C = lerp(C, c0, t);
	}
	
	c0 = gSteepMap.Sample(gAnisotropicSam,psIn.tiledUV);
	C=lerp(c0,C,slope);

    C.rgb *= psIn.shade;

	float shadowFactor = CalcShadowFactor(psIn.projTexC);
	C.rgb *= saturate(shadowFactor+0.3);

	float fog = CalcFogFactor( psIn.fogDist , float3(1,0,0));
	C.rgb = lerp(g_fogColor, C.rgb, fog);

    return C;
}

technique10 TerrainTech
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
    }
}
