#include "Fog.fx"
#include "ShadowMap.fx"


float3 gMaterialAmbient   = float3( 0.2f, 0.2f, 0.2f );
float3 gMaterialDiffuse   = float3( 0.8f, 0.8f, 0.8f );
float3 gMaterialSpecular  = float3( 1.0f, 1.0f, 1.0f );
float  gMaterialAlpha     = 1.0f;
int    gMaterialShininess = 32;

float3 gLightColor        = float3( 1.0f, 1.0f, 1.0f );
float3 gLightPosition     = float3( 0.0f, 1.0f, 0.0f );
float3 gCameraPosition ;

Texture2D gMeshTexture;        // Color texture for mesh

float  g_fTime ;                // App's time in seconds
float4x4 g_mWorld ;               // World matrix
float4x4 gWVP ; // World * View * Projection matrix
float4x4 gView;


RasterizerState NoCull
{
    CullMode = None;
};


SamplerState samLinear
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};


struct VS_IN
{
    float3 vPosObject   : POSITION;
    float3 vNormalObject: NORMAL;
    float2 vTexUV       : TEXCOORD;
};

struct PS_IN
{
	
    float4 vPosProj : SV_POSITION;
    //float4 vColor   : COLOR0;
	float3 LightVec	: TEXCOORD1;
	float3 WorldNormal : TEXCOORD2;
	float3 CameraVec : TEXCOORD3;
    float2 vTexUV   : TEXCOORD0;
	float  fogDist : FOGDISTANCE;
	float4 projTexC : TEXCOORD4;
};

PS_IN VS( VS_IN vsIn)
{
    PS_IN vsOut;
    
    // Transform the position into world space for lighting, and projected space
    // for display
    float4 vPosWorld = mul( float4(vsIn.vPosObject,1), g_mWorld );
    vsOut.vPosProj = mul( float4(vsIn.vPosObject,1), gWVP );

    // Transform the normal into world space for lighting
    vsOut.WorldNormal = mul( vsIn.vNormalObject, (float3x3)g_mWorld );

    // Pass the texture coordinate
    vsOut.vTexUV = vsIn.vTexUV;

    // Compute the light vector
    vsOut.LightVec = normalize( gLightPosition - vPosWorld.xyz );

	vsOut.CameraVec = vPosWorld.xyz - gCameraPosition;
	
	vsOut.fogDist = mul( vPosWorld, gView ).z; //save cameraPos for fog calculation

	// Generate projective tex-coords to project shadow map onto scene.
	vsOut.projTexC = mul(float4(vsIn.vPosObject, 1.0f), gLightWVP);
    return vsOut;
}

float4 PS( PS_IN psIn, uniform bool bTexture, uniform bool bSpecular) : SV_Target
{
	float3 Ln = normalize(psIn.LightVec);
	float3 Nn = normalize(psIn.WorldNormal);

    // Compute the ambient and diffuse components of illumination
	// Sample and modulate the texture
	float4 psOut = float4(0,0,0,1);
	
	if ( bTexture )
    {
		float4 diffusec = gMeshTexture.Sample( samLinear, psIn.vTexUV );
		clip(diffusec.a - 0.15f);
		psOut.rgb = diffusec * 0.13;
        psOut.rgb += diffusec * saturate( dot( Ln, Nn )) * 0.65;
		psOut.a = diffusec.a;

    } else {
		psOut.rgb = gLightColor * gMaterialAmbient;
		psOut.rgb += gLightColor * gMaterialDiffuse * saturate( dot( Ln, Nn ) );
		psOut.a = gMaterialAlpha;
	}
    
	
	/*
    // If enabled, calculate the specular term
    if ( bSpecular )
    {
        //float3 vCamera = normalize( vPosWorld.xyz - gCameraPosition );
		float3 Cn = normalize(psIn.CameraVec);
		
        float3 vReflection = reflect( Ln, Nn );
        float  fPhoneValue = saturate( dot(vReflection, Cn) );
		
        psOut.rgb += gMaterialSpecular * pow( fPhoneValue, gMaterialShininess );
		
    }
	*/
    

	float shadowFactor = CalcShadowFactor(psIn.projTexC);
	psOut.rgb *= saturate(shadowFactor+0.3);

	float fog = CalcFogFactor( psIn.fogDist, psIn.CameraVec );
	psOut.rgb = lerp(g_fogColor, psOut.rgb, fog);

    return psOut;
}

technique10 NoSpecular
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS(false, false) ) );

		SetRasterizerState(NoCull);
    }
}
technique10 Specular
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS(false,true) ) );

		SetRasterizerState(NoCull);
    }
}
technique10 TexturedNoSpecular
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS(true,false) ) );

		SetRasterizerState(NoCull);
    }
}
technique10 TexturedSpecular
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS(true,true) ) );

		SetRasterizerState(NoCull);
    }
}
