Texture2D gShadowMap;

cbuffer cbPerObject
{
	float4x4 gLightWVP;
}

SamplerState gShadowSam
{
	Filter = MIN_MAG_MIP_POINT;
	AddressU = Clamp;
	AddressV = Clamp;
};


#define SHADOW_RES 2048.0f
#define SHADOW_RES_REC (1.0f / SHADOW_RES)
#define SHIFT 0.003f

float CalcShadowFactor(float4 projTexC)
{
	//complete projection
	projTexC.xyz /= projTexC.w;
	
	//points outside the shadow range
	if( abs(projTexC.x) > 1.0f || abs(projTexC.y) > 1.0f)
		return 1.0f;
	  
	//NDC -> texture space
	projTexC.x = +0.5f * projTexC.x + 0.5f;
	projTexC.y = -0.5f * projTexC.y + 0.5f;
	
	//sample 4 times (get a 4x4 area)
	float s0 = gShadowMap.Sample(gShadowSam, projTexC.xy).r;
	float s1 = gShadowMap.Sample(gShadowSam, projTexC.xy + float2(SHADOW_RES_REC, 0)).r;
	float s2 = gShadowMap.Sample(gShadowSam, projTexC.xy + float2(0, SHADOW_RES_REC)).r;
	float s3 = gShadowMap.Sample(gShadowSam, projTexC.xy + float2(SHADOW_RES_REC, SHADOW_RES_REC)).r;
	
	float sampleDepth = projTexC.z;

	//do the shadow tests
	float result0 = sampleDepth <= s0 + SHIFT;
	float result1 = sampleDepth <= s1 + SHIFT;
	float result2 = sampleDepth <= s2 + SHIFT;
	float result3 = sampleDepth <= s3 + SHIFT;	
	
	//get a difference between the center of the texture and the sampled coordinates
	float2 texelPos = frac(SHADOW_RES*projTexC.xy);

	// Interpolate results.
	return lerp( lerp(result0, result1, texelPos.x), 
	             lerp(result2, result3, texelPos.x), texelPos.y);		 
}
